'use strict';

define([
	'app',
	'libraries/language',
	'libraries/consts'
	], function(app, LANGUAGE, CONSTS) {
	app.controller("accessTokenManage",
		["$rootScope", "$scope", "$routeParams", "$location", "$timeout", "$uibModal", "$q", "jetapi", "meta", "utils", "SweetAlert",
			function ($rootScope, $scope, $routeParams, $location, $timeout, $uibModal, $q, jetapi, meta, utils, SweetAlert) {
				$rootScope.$emit('menuItem', 'Users');

				$scope.token_user = $routeParams.id;
				$scope.tokens = [];
				$scope.saveData = {
					ip: '',
					description: ''
				};

				$scope.saving = false;

				$scope.status = undefined;
				$scope.cancelled = false;
				$scope.changed = true;
				$scope.loadingTokens = false;

				$scope.$on('$destroy', function() {
					if(!$scope.changed || $scope.cancelled) return;

					SweetAlert.swal({
						title: LANGUAGE.translate("Are you sure?"),
						text: LANGUAGE.translate("You didn't saved your changes"),
						icon: "warning",
						buttons: [LANGUAGE.translate("Disregard Changes"), LANGUAGE.translate("Save")]
					}, function(isConfirm){
						if (isConfirm) {
							$scope.saveChanges();
						}
					});
				});

				$scope.cancel = function() {
					$scope.cancelled = true;
					$location.path('/accessTokens/' + $routeParams.id);
				};

				$scope.saveChanges = function() {

					if($scope.saving) return;

					$scope.status = undefined;
					$scope.saving = true;

					$scope.saveData.user = $routeParams.id;

					jetapi.manageAccessToken($scope.saveData, function(response) {

						$scope.changed = false;
						$scope.saving = false;

						if(response.success)
						{
							$location.path('/accessTokens/' + $routeParams.id);
							SweetAlert.swal(LANGUAGE.translate("Success!"), LANGUAGE.translate(response.message), "success");
						}
						else
						{
							$scope.status = {
								message: LANGUAGE.translate(response.message),
								type: response.success ? "success" : "danger",
								closeable: false,
								ttl: 10000
							};
						}
					});
				};
	}]);
});