'use strict';

define([
	'app',
	'libraries/utils',
	'libraries/language',
	'libraries/permissions',
	'libraries/consts'
	], function(app, UTILS, LANGUAGE, PERMISSIONS, CONSTS) {
	app.controller("backupManagerDatabases",
		["$rootScope", "$scope", "$routeParams", "$location", "$timeout", "$uibModal", "$q", "jetapi", "meta", "utils", "SweetAlert",
			function ($rootScope, $scope, $routeParams, $location, $timeout, $uibModal, $q, jetapi, meta, utils, SweetAlert) {

				$scope.databases = [];
				$scope.database_data = {};
				$scope.backups = {};
				$scope.downloads = {};
				$scope.databases_loaded = false;
				$scope.loadingDatabases = false;
				$scope.checked = false;
				$scope.status = undefined;

				meta = meta.new("backup_manager_databases");
				$scope.meta = meta;
				$scope.metaData = meta.getData();

				if(!meta.getSortBy()) meta.setSortBy("name");
				if(!meta.getSortDirection()) meta.setSortDirection("asc");
				meta.setSortFields(["name","type"]);
				meta.setTotalItems($scope.databases.length);

				$scope.destinations = [{_id: '', name: LANGUAGE.translate('All') }];

				$scope.flags = [
					{ label: LANGUAGE.translate("All"), value: 0 },
					{ label: LANGUAGE.translate("Incremental"), value: 1 },
					{ label: LANGUAGE.translate("Archived"), value: 2 },
					{ label: LANGUAGE.translate("Compressed"), value: 4 }
				];

				$scope.engines = [
					{ label: "MySQL", value: 'MySQL' },
					{ label: "PostgreSQL", value: 'PostgreSQL' },
					{ label: "MongoDB", value: 'MongoDB' }
				];

				$scope.filter = {
					destination: '',
					engine: "MySQL"
				};

				var toggleLock = function(backup, lock, ttl, callback) {
					jetapi.manageBackup({ _id: backup._id, lock_ttl: ttl, locked: lock }, function(response) {
						if(callback !== undefined) callback(response, backup);
					});
				};

				$scope.lockSelected = function() {
					$scope.lockTimeSelection(function (ttl) {
						for (var database in $scope.database_data) {
							if ($scope.database_data[database].checked) {
								toggleLock($scope.backups[database][$scope.database_data[database].selected_id], 1, ttl, function (response, backup) {
									if (response.success) backup.locked = 1;
								});
							}
						}

						SweetAlert.success(LANGUAGE.translate("Success!"), LANGUAGE.translate("The selected backups locked successfully"));
					});
				};

				$scope.unlockSelected = function() {
					for(var database in $scope.database_data) {
						if($scope.database_data[database].checked)
						{
							toggleLock($scope.backups[database][$scope.database_data[database].selected_id], 0, 0, function (response, backup) {
								if(response.success) backup.locked = 0;
							});
						}
					}

					SweetAlert.success(LANGUAGE.translate("Success!"), LANGUAGE.translate("The selected backups unlocked successfully"));
				};

				$scope.toggleLock = function(backup) {
					var status = !backup.locked;
					$scope.lockTimeSelection(function (ttl) {
						toggleLock(backup, status, ttl, function (response, backup) {

							if (response.success) {
								backup.locked = status;
								SweetAlert.success(LANGUAGE.translate("Success!"), LANGUAGE.translate(response.message));
								return;
							}

							SweetAlert.error(LANGUAGE.translate("Error!"), LANGUAGE.translate(response.message));
						});
					}, !status);
				};

				$scope.editNotes = function (backup) {
					backup.editing = true;
				};

				$scope.saveNotes = function (backup) {
					jetapi.manageBackup({ _id: backup._id, notes: backup.notes });
					backup.editing = false;
				};

				$scope.showDatabaseBackup = function(database) {
					return $scope.database_data[database.identity] !== undefined && $scope.database_data[database.identity].selected_id !== undefined;
				};

				$scope.setCheckAll = function() {
					for(var i = 0; i < $scope.databases.length; i++) {
						if(!$scope.database_data[$scope.databases[i].identity].restore && !$scope.database_data[$scope.databases[i].identity].download) {
							$scope.database_data[$scope.databases[i].identity].checked = $scope.checkall;
						}
					}
					$scope.checkChanged();
				};

				$scope.checkChanged = function () {
					for(var database in $scope.database_data) {
						if($scope.database_data[database].checked)
						{
							$scope.checked = true;
							return;
						}
					}

					$scope.checked = false;
				};

				$scope.checkDatabase = function (database) {
					if($scope.database_data[database.identity].selected_id === undefined || $scope.database_data[database.identity].restore || $scope.database_data[database.identity].download) return;
					$scope.database_data[database.identity].checked = !$scope.database_data[database.identity].checked;
					$scope.checkChanged();
				};

				$scope.restoreSelected = function () {

					$scope.status = undefined;

					var modalInstance = $uibModal.open({
						animation: true,
						size: "lg",
						ariaLabelledBy: 'modal-title',
						ariaDescribedBy: 'modal-body',
						templateUrl: './app/views/databaseServerSelection.ptt',
						controller: 'databaseServerSelection',
						controllerAs: '$scope',
						resolve: {
							engine: function() { return $scope.filter.engine; }
						}
					});

					modalInstance.result.then(function(server_id) {

						if(!server_id) {
							$scope.status = {
								message: LANGUAGE.translate("You didn't selected any database server to restore to."),
								type: "danger",
								closeable: false,
								ttl: 10000
							};
							return;
						}
						var failed = ["<br />"];
						for(var database in $scope.database_data) {
							if($scope.database_data[database].checked)
							{
								$scope.selectedDatabase = database;

								jetapi.addQueueRestore({ _id: $scope.database_data[database].selected_id, database: server_id }, function(response, params) {
									$scope.database_data[params.database].checked = false;

									if(response.success) {
										$scope.backups[params.database][$scope.database_data[params.database].selected_id].queue.restore = response.data.queue_id;
										$scope.database_data[params.database].restore = response.data.queue_id;
									} else {
										console.log($scope.database_data[database]);
										failed.push(LANGUAGE.translate("Database \"%s\"", $scope.database_data[params.database].name) + ": " + response.message);

										$scope.status = {
											message: failed.join("<br />"),
											type: "danger",
											closeable: false,
											ttl: 10000
										};
									}
								}, {database: database});
							}
						}

						$scope.checkall = false;
						$scope.checked = false;
						SweetAlert.success(LANGUAGE.translate("Success!"), LANGUAGE.translate("Your request to add the selected databases to the restore queue finished successfully. Please check the top of this page to see is there was errors while processing this request."));

					}, function () {});
				};

				$scope.downloadSelected = function () {

					$scope.status = undefined;

					SweetAlert.swal({
						title: LANGUAGE.translate("Are you sure?"),
						text: LANGUAGE.translate("The selected backups will be added to the download queue."),
						icon: "warning",
						buttons: [LANGUAGE.translate("Cancel"), true]
					}, function(isConfirm){

						if (isConfirm)
						{
							var failed = ["<br />"];

							for(var database in $scope.database_data) {
								if($scope.database_data[database].checked)
								{
									jetapi.addQueueDownload({ _id: $scope.database_data[database].selected_id }, function(response, params) {
										$scope.database_data[params.database].checked = false;

										if(response.success && response.data.added) {
											$scope.backups[params.database][$scope.database_data[params.database].selected_id].queue.download = response.data._id;
											$scope.database_data[params.database].download = response.data._id;
										} else {
											failed.push(LANGUAGE.translate("Database \"%s\"", $scope.database_data[params.database].name) + ": " + LANGUAGE.translate(response.message));

											$scope.status = {
												message: failed.join("<br />"),
												type: "danger",
												closeable: false,
												ttl: 10000
											};
										}
									}, {database: database});
								}
							}

							$scope.checkall = false;
							$scope.checked = false;
							SweetAlert.success(LANGUAGE.translate("Success!"), LANGUAGE.translate("Your request to add the selected databases to the download queue finished successfully. Please check the top of this page to see is there was errors while processing this request."));
						}
						else
						{
							SweetAlert.error(LANGUAGE.translate("Cancelled!"), LANGUAGE.translate("Download cancelled"));
						}

					});
				};

				$scope.$watch("filter", function(){
					if(!$scope.databases_loaded) return;

					$scope.checkall = false;
					$scope.resetBackupList();
					$scope.fetchData();
				}, true);


				$scope.$on('fetch', $scope.fetchData);

				$scope.fetchData = function() {

					$scope.loadingDatabases = true;

					var apiParams = {
						sort: {},
						skip: meta.getSkip(),
						limit: meta.getPageSize(),
						filter: meta.getFilter(),
						find: {}
					};

					apiParams.sort[meta.getSortBy()] = meta.getSortDirectionInt();

					if($scope.filter.engine) apiParams.find.params = { engine: $scope.filter.engine };

					$scope.databases = [];

					jetapi.listDatabases(apiParams, function (response) {
						meta.setTotalItems(response.data.total);
						meta.calculate(response.data.databases);
						$scope.databases = response.data.databases;

						for(var i = 0; i < $scope.databases.length; i++) {
							$scope.databases[i].identity = $scope.databases[i].name + $scope.databases[i].type;
						}

						$scope.databases_loaded = true;
						$scope.listBackups();
						$scope.loadingDatabases = false;
					});
				};

				$scope.calculatingBackupSize = '';
				$scope.calculateBackupSize = function (database, _id) {

					if($scope.calculatingBackupSize) return;

					$scope.calculatingBackupSize = _id;

					jetapi.calculateBackupSize({ _id: _id }, function(response) {
						$scope.calculatingBackupSize = '';
						if(!response.success) {
							SweetAlert.error(LANGUAGE.translate("Error!"), LANGUAGE.translate(response.message));
							return;
						}
						$scope.backups[database][_id].size = response.data.size;
					});
				};

				$scope.listBackups = function(callback) {

					$scope.resetBackupList();
					$scope.loadingDatabases = true;

					var apiParams = {
						type: CONSTS.BACKUP_TYPE_DATABASES,
						sort: { created: -1 },
						find: {},
						databases: []
					};

					if($scope.filter.destination) apiParams.find.dest = $scope.filter.destination;
					for(var i = 0; i < $scope.databases.length; i++) apiParams.databases.push($scope.databases[i].name);

					jetapi.listBackups(apiParams, function (response) {

						for(var i = 0; i < response.data.backups.length; i++)
						{
							var backup = response.data.backups[i];
							var database = backup.name + backup.type;

							backup.display = LANGUAGE.date(backup.created) + (backup.encrypted ? ' - ' + LANGUAGE.translate('Encrypted') : '');

							if($scope.backups[database] === undefined) $scope.backups[database] = {};
							$scope.backups[database][backup._id] = backup;

							if($scope.database_data[database] === undefined) $scope.database_data[database] = { checked: false, restore: false, download: false, name: backup.name };
							$scope.database_data[database].name = backup.name;
							if($scope.database_data[database].selected_id === undefined) $scope.database_data[database].selected_id = backup._id;
							if(backup.queue.restore) $scope.database_data[database].restore = backup.queue.restore;
							if(backup.queue.download) $scope.database_data[database].download = backup.queue.download;
						}

						$scope.loadingDatabases = false;
						if(callback !== undefined) callback(response);

					});
				};

				$scope.resetBackupList = function() {

					if(!$scope.databases_loaded) return;

					$scope.checked = false;
					$scope.backups = {};
					$scope.database_data= {};

					for(var i = 0; i < $scope.databases.length; i++) {
						if($scope.database_data[$scope.databases[i].identity] === undefined) $scope.database_data[$scope.databases[i].identity] = { checked: false, restore: false, download: false };
						$scope.backups[$scope.databases[i].identity] = {};
					}
				};

				$scope.fetchDestinations = function() {

					jetapi.listDestinations({}, function(response) {

						if(!response.success) return;

						for(var i = 0; i < response.data.destinations.length; i++)
						{
							if(response.data.destinations[i].clone_destination) continue;
							$scope.destinations.push({
								_id: response.data.destinations[i]._id,
								name: response.data.destinations[i].name + " (" + response.data.destinations[i].type + ")"
							});
						}

					});
				};

				$timeout($scope.fetchDestinations());
				$timeout($scope.fetchData());
			}]);
});