define([
	'app',
	'libraries/language',
	'libraries/utils'
], function(app, LANGUAGE, UTILS) {
	app.controller("dashboard",
		["$rootScope", "$scope", "$location", "$timeout", "$interval", "jetapi", "meta",
			function($rootScope, $scope, $location, $timeout, $interval, jetapi, meta) {
				$rootScope.$emit('menuItem', 'Dashboard');

				$scope.stats = {
					alerts: 0,
					accounts: 0,
					jobs: 0,
					backups: 0,
					over_quota: 0,
					jobs_running: 0
				};

				$scope.loadingStatistics = true;
				$scope.loadingDestinations = false;
				$scope.loadingBackups = false;
				$scope.loadingQueues = false;
				$scope.loadingAlerts = false;
				$scope.loadingAccountsWithoutBackupJob = false;
				$scope.resourcesTimeout = null;

				$scope.restore_queue_summary = {};
				$scope.alerts = [];
				$scope.destinations = [];
				$scope.backups = [];
				$scope.queues = [];
				$scope.accounts = [];
				$scope.interval = undefined;
				$scope.disks = {};
				$scope.networks = [];
				$scope.ios = [];
				$scope.memory = {};
				$scope.cpu = {};

				$scope.disksChart = {
					labels: [],
					data: [[]],
					options: {
						responsive: true,
						maintainAspectRatio: false,
						tooltips: {
							mode: 'index',
							intersect: false,
							enabled: false,
							custom: function(tooltipModel) {
								var tooltipEl = document.getElementById('disks-tooltip');

								// Create element on first render
								if (!tooltipEl) {
									tooltipEl = document.createElement('div');
									tooltipEl.id = 'disks-tooltip';
									tooltipEl.innerHTML = "<div style='background: #fff; padding: 5px; border-radius: 5px; border: 1px solid #E6E9ED;'></div>";
									document.body.appendChild(tooltipEl);
								}

								if (tooltipModel.opacity === 0) {
									tooltipEl.style.opacity = 0;
									return;
								}

								tooltipEl.classList.remove('above', 'below', 'no-transform');
								if (tooltipModel.yAlign) {
									tooltipEl.classList.add(tooltipModel.yAlign);
								} else {
									tooltipEl.classList.add('no-transform');
								}

								if (tooltipModel.body) {
									var titleLines = tooltipModel.title || [];
									var bodyLines = tooltipModel.body.map(function (bodyItem) {
										return bodyItem.lines;
									});

									var disk = $scope.disks[titleLines[0]];

									var innerHtml = '';
									innerHtml += '<div style="font-size: 14px; font-weight: bold;">' + titleLines[0] + ' <span style="font-size: 10px;">(' + disk.filesystem + ')</span></div>';

									bodyLines.forEach(function(body, i) {
										innerHtml += '<div>';
										innerHtml += '<strong>Total</strong>: ' + UTILS.sizeToHumanReadable(disk.max) + '<br />';
										innerHtml += '<strong>Used</strong>: ' + UTILS.sizeToHumanReadable(disk.usage) + ' (' + Math.round(disk.percentage) + '%)<br />';
										innerHtml += '<strong>Available</strong>: ' + UTILS.sizeToHumanReadable(disk.max - disk.usage) + ' (' + (100 - Math.round(disk.percentage)) + '%)';
										innerHtml += '</div>';
									});

									var tableRoot = tooltipEl.querySelector('div');
									tableRoot.innerHTML = innerHtml;
								}

								var position = this._chart.canvas.getBoundingClientRect();

								tooltipEl.style.opacity = 1;
								tooltipEl.style.position = 'absolute';
								tooltipEl.style.left = position.left + window.pageXOffset + tooltipModel.caretX + 'px';
								tooltipEl.style.top = position.top + window.pageYOffset + tooltipModel.caretY + 'px';
								tooltipEl.style.fontFamily = tooltipModel._bodyFontFamily;
								tooltipEl.style.fontSize = tooltipModel.bodyFontSize + 'px';
								tooltipEl.style.fontStyle = tooltipModel._bodyFontStyle;
								tooltipEl.style.padding = tooltipModel.yPadding + 'px ' + tooltipModel.xPadding + 'px';
								tooltipEl.style.pointerEvents = 'none';
							}
						},
						hover: {
							mode: 'index',
							intersect: false
						},
						scales: {
							xAxes: [{
								display: true,
								ticks: {
									beginAtZero: true,
									steps: 100,
									stepValue: 1,
									max: 100
								}
							}]
						}
					}
				};

				$scope.fetchResources = function() {

					jetapi.getServerResources({ withLoader: false }, function (response) {

						$scope.resourcesTimeout = setTimeout($scope.fetchResources, 3000);
						if(!response.success) return;

						$scope.networks = response.data.networks;
						$scope.ios = response.data.io;
						$scope.memory = response.data.memory;
						$scope.memory.percentage = Math.round($scope.memory.percentage);

						$scope.cpu = response.data.cpu;
						$scope.cpu.percentage = Math.round($scope.cpu.percentage);

						$scope.disks = [];
						$scope.disksChart.labels = [];
						$scope.disksChart.data = [[]];

						for(var i = 0; i < response.data.disks.length; i++) {
							var disk = response.data.disks[i];
							$scope.disksChart.labels.push(disk.partition);
							$scope.disksChart.data[0].push(Math.round(disk.percentage));
							$scope.disks[disk.partition] = disk;
						}
					});

				};

				$scope.fetchResources();

				$scope.destinationMeta = meta.new("dashboard_destinations");
				$scope.destinationMetaData = $scope.destinationMeta.getData();
				$scope.destinationMeta.setSortBy("jobs_count");
				$scope.destinationMeta.setSortDirection("asc");
				$scope.destinationMeta.setTotalItems($scope.destinations.length);
				$scope.destinationMeta.setPageSizes([5,10,25]);
				$scope.destinationMeta.setPageSize(5);
				$scope.destinationMeta.setLimit(5);
				$scope.destinationMeta.setMaxPages(1);

				$scope.backupMeta = meta.new("dashboard_backups");
				$scope.backupMetaData = $scope.backupMeta.getData();
				$scope.backupMeta.setSortBy("next_run");
				$scope.backupMeta.setSortDirection("asc");
				$scope.backupMeta.setTotalItems($scope.backups.length);
				$scope.backupMeta.setPageSizes([5,10,25]);
				$scope.backupMeta.setPageSize(5);
				$scope.backupMeta.setLimit(5);
				$scope.backupMeta.setMaxPages(1);

				$scope.queueMeta = meta.new("dashboard_queues");
				$scope.queueMetaData = $scope.queueMeta.getData();
				$scope.queueMeta.setSortBy("created");
				$scope.queueMeta.setSortDirection("desc");
				$scope.queueMeta.setTotalItems($scope.queues.length);
				$scope.queueMeta.setPageSizes([5,10,25]);
				$scope.queueMeta.setPageSize(5);
				$scope.queueMeta.setLimit(5);

				$scope.fetchData = function(callback) {
					jetapi.getDashboardDetails({}, function(response) {
						$scope.stats.alerts = response.data.statistics.total_acknowledged_alerts;
						$scope.stats.accounts = response.data.statistics.total_accounts;
						$scope.stats.jobs = response.data.statistics.total_backup_jobs;
						$scope.stats.backups = response.data.statistics.total_backups;
						$scope.stats.disk_usage = response.data.statistics.total_disk_usage;
						$scope.stats.jobs_running = response.data.statistics.total_backup_jobs_running;
						$scope.alerts = response.data.alerts.alerts;
						$scope.restore_queue_summary = response.data.restore_queue_summary;

						$scope.destinationMeta.setTotalItems(response.data.statistics.total_destinations);
						$scope.destinationMeta.calculate(response.data.destinations.destinations);
						$scope.destinations = response.data.destinations.destinations;

						$scope.backupMeta.setTotalItems(response.data.backup_jobs.total);
						$scope.backupMeta.calculate(response.data.backup_jobs.jobs);
						$scope.backups = response.data.backup_jobs.jobs;

						$scope.queueMeta.setTotalItems(response.data.queue.total);
						$scope.queueMeta.calculate(response.data.queue.queue);
						$scope.queues = response.data.queue.queue;

						if(callback !== undefined) callback(response);
					});
				};

				$scope.$on('$destroy', function() {
					if($scope.resourcesTimeout !== null) {
						clearTimeout($scope.resourcesTimeout);
						$scope.resourcesTimeout = null;
					}
				});

				function fetch() {
					$scope.loadingStatistics = true;
					$scope.fetchData(function() {
						$scope.loadingStatistics = false;
					});
				}

				$scope.listDestinations = function() {

					$scope.loadingDestinations = true;
					$scope.destinations = [];

					var sort = {};
					sort[$scope.destinationMeta.getSortBy()] = $scope.destinationMeta.getSortDirectionInt();

					jetapi.listDestinations({
						sort: sort,
						skip: $scope.destinationMeta.getSkip(),
						limit: $scope.destinationMeta.getLimit()
					}, function(response) {
						$scope.destinationMeta.setTotalItems(response.data.total);
						$scope.destinationMeta.calculate(response.data.destinations);
						$scope.destinations = response.data.destinations;
						$scope.loadingDestinations = false;
					});
				};

				$scope.listBackupJobs = function() {

					$scope.loadingBackups = true;
					$scope.backups = [];

					var sort = {};
					sort[$scope.backupMeta.getSortBy()] = $scope.backupMeta.getSortDirectionInt();

					jetapi.listBackupJobs({
						sort: sort,
						skip: $scope.backupMeta.getSkip(),
						limit: $scope.backupMeta.getLimit()
					}, function(response) {
						$scope.backupMeta.setTotalItems(response.data.total);
						$scope.backupMeta.calculate(response.data.jobs);
						$scope.backups = response.data.jobs;
						$scope.loadingBackups = false;
					});
				};

				$scope.listQueueItems = function() {

					$scope.loadingQueues = true;
					$scope.queues = [];

					var sort = {};
					sort[$scope.queueMeta.getSortBy()] = $scope.queueMeta.getSortDirectionInt();

					jetapi.listQueueItems({
						sort: sort,
						skip: $scope.queueMeta.getSkip(),
						limit: $scope.queueMeta.getLimit()
					}, function(response) {
						$scope.queueMeta.setTotalItems(response.data.total);
						$scope.queueMeta.calculate(response.data.queue);
						$scope.queues = response.data.queue;
						$scope.loadingQueues = false;
					});
				};

				$timeout(fetch);
			}
		]
	);
});