'use strict';

define([
	'app',
	'libraries/language',
	'libraries/consts'
	], function(app, LANGUAGE, CONSTS) {
	app.controller("databaseFilterGroups",
		["$rootScope", "$scope", "$location", "$timeout", "jetapi", "meta", "SweetAlert",
			function ($rootScope, $scope, $location, $timeout, jetapi, meta, SweetAlert) {
				$rootScope.$emit('menuItem', 'DatabaseFilters');

				$scope.groups = [];
				$scope.filters = {};
				$scope.loadingGroups = false;
				$scope.saveing = false;
				$scope.status = undefined;

				$scope.types = {};
				$scope.types[CONSTS.ACCOUNT_FILTER_TYPE_ACCOUNT] = LANGUAGE.translate("Databases Filter");
				$scope.types[CONSTS.ACCOUNT_FILTER_TYPE_RESELLER] = LANGUAGE.translate("Resellers Filter");
				$scope.types[CONSTS.ACCOUNT_FILTER_TYPE_SUSPENSION] = LANGUAGE.translate("Suspension Filter");
				$scope.types[CONSTS.ACCOUNT_FILTER_TYPE_DISK_USAGE] = LANGUAGE.translate("Disk Space Usage Filter");
				$scope.types[CONSTS.ACCOUNT_FILTER_TYPE_INODE_USAGE] = LANGUAGE.translate("Inodes Usage Filter");
				$scope.types[CONSTS.ACCOUNT_FILTER_TYPE_PACKAGE] = LANGUAGE.translate("Packages Filter");
				$scope.types[CONSTS.ACCOUNT_FILTER_TYPE_RANGE] = LANGUAGE.translate("Characters Range Filter");
				$scope.types[CONSTS.ACCOUNT_FILTER_TYPE_REGEX] = LANGUAGE.translate("Regular Expression Filter");

				$scope.loaders = {
					delete: false
				};

				meta = meta.new("filter_groups");
				$scope.meta = meta;
				$scope.metaData = meta.getData();

				if(!meta.getSortBy()) meta.setSortBy("name");
				if(!meta.getSortDirection()) meta.setSortDirection("desc");
				meta.setSortFields(["name","type","owner","jobs_count"]);
				meta.setTotalItems($scope.groups.length);

				$scope.onClickDelete = function(group) {

					if($scope.saveing) return;

					if(group.jobs_count > 0)
					{
						SweetAlert.error(LANGUAGE.translate("Failed!"), LANGUAGE.translate("This database filter group has an assigned jobs, Please remove those jobs before deleting this database filter group."));
						return false;
					}

					$scope.saveing = true;

					SweetAlert.swal({
						title: LANGUAGE.translate("Are you sure?"),
						text: LANGUAGE.translate("This group will be permanently deleted!"),
						icon: "warning",
						buttons: [LANGUAGE.translate("Cancel"), true]
					}, function(isConfirm){

						if (isConfirm)
						{
							$scope.loaders.delete = true;

							jetapi.deleteDatabaseFilterGroup({ _id: group._id }, function(response) {

								$scope.saveing = false;
								$scope.loaders.delete = false;

								if(response.success)
								{
									$scope.fetchData();
									SweetAlert.success(LANGUAGE.translate("Success!"), LANGUAGE.translate(response.message));
								}
								else
								{
									SweetAlert.error(LANGUAGE.translate("Failed!"), LANGUAGE.translate(response.message));
								}

							});
						}
						else
						{
							$scope.saveing = false;
							SweetAlert.error(LANGUAGE.translate("Cancelled!"), LANGUAGE.translate("Group delete cancelled"));
						}

					});
				};

				$scope.fetchFilters = function (callback) {

					$scope.filters = {};
					jetapi.listDatabaseFilters({}, function (response) {
						for(var i = 0; i < response.data.filters.length; i++) {
							$scope.filters[response.data.filters[i]._id] = response.data.filters[i];
						}
						if(callback !== undefined) callback();
					});
				};

				$scope.$on('fetch', $scope.fetchData);

				$scope.fetchData = function() {

					$scope.loadingGroups = true;
					$scope.status = undefined;

					var apiParams = {
						sort: {},
						skip: meta.getSkip(),
						limit: meta.getPageSize(),
						find: {},
						filter: meta.getFilter()
					};

					apiParams.sort[meta.getSortBy()] = meta.getSortDirectionInt();

					$scope.groups = [];

					jetapi.listDatabaseFilterGroups(apiParams, function(response) {
						meta.setTotalItems(response.data.total);
						meta.calculate(response.data.groups);
						$scope.groups = response.data.groups;
						$scope.loadingGroups = false;
					});
				};

				$timeout($scope.fetchFilters(function() {
					$scope.fetchData();
				}));
			}
		]
	);
});