'use strict';

define([
	'app',
	'libraries/language',
	'libraries/permissions',
	'libraries/consts'
	], function(app, LANGUAGE, PERMISSIONS, CONSTS) {
	app.controller("databaseFilterManage",
		["$rootScope", "$scope", "$routeParams", "$location", "$timeout", "$uibModal", "$q", "jetapi", "meta", "utils", "SweetAlert",
			function ($rootScope, $scope, $routeParams, $location, $timeout, $uibModal, $q, jetapi, meta, utils, SweetAlert) {
				$rootScope.$emit('menuItem', 'DatabaseFilters');

				$scope.details = {
					name: ''
				};

				$scope.saveing = false;
				$scope.status = undefined;
				$scope.changed = false;
				$scope.cancelled = false;
				$scope.range = "0123456789_abcdefghijklmnopqrstuvwxyz";
				$scope.filterData = {
					list: ''
				};

				$scope.conditions = [
					{ label: LANGUAGE.translate("Include"), value: CONSTS.DATABASE_FILTER_CONDITION_INCLUDE },
					{ label: LANGUAGE.translate("Exclude"), value: CONSTS.DATABASE_FILTER_CONDITION_EXCLUDE }
				];

				$scope.types = [
					{ label: LANGUAGE.translate("Databases Filter"), value: CONSTS.DATABASE_FILTER_TYPE_DATABASE, template: 'database' },
					{ label: LANGUAGE.translate("Characters Range Filter"), value: CONSTS.DATABASE_FILTER_TYPE_RANGE, template: 'range' },
					{ label: LANGUAGE.translate("Regular Expression Filter"), value: CONSTS.DATABASE_FILTER_TYPE_REGEX, template: 'regex' }
				];

				$scope.loadFilterType = function(type, defaults) {

					$scope.saveData = utils.duplicateObject($scope.details);

					if(defaults)
					{
						$scope.saveData.condition = CONSTS.DATABASE_FILTER_CONDITION_INCLUDE;
						$scope.saveData.type = type;

						switch(type)
						{
							case CONSTS.DATABASE_FILTER_TYPE_DATABASE:
								$scope.saveData.list = [];
							break;

							case CONSTS.DATABASE_FILTER_TYPE_RANGE:
								$scope.saveData.rangestart = '0';
								$scope.saveData.rangeend = 'z';
							break;

							case CONSTS.DATABASE_FILTER_TYPE_REGEX:
								$scope.saveData.regex = '';
							break;
						}
					}

					$scope.conditions[0].label = LANGUAGE.translate("Include");
					$scope.conditions[1].label = LANGUAGE.translate("Exclude");

					for(var i = 0; i < $scope.types.length; i++) {
						if($scope.types[i].value === type) {
							$scope.filterType = './app/views/databaseFilters/' + $scope.types[i].template + '.ptt';
							return;
						}
					}

				};

				$scope.validateName = function (name) {
					return true;
					if(!name.trim()) return false;
					return !CONSTS.DATABASE_NAME_FILTER.test(name);
				};

				$scope.addListRow = function () {
					if($scope.filterData.list === undefined || !$scope.filterData.list.trim()) return;
					if($scope.validateName($scope.filterData.list.trim())) $scope.saveData.list.push($scope.filterData.list.trim());
					else SweetAlert.error(LANGUAGE.translate("Error!"), LANGUAGE.translate("The provided database (\"%s\") has invalid name.", $scope.filterData.list.trim()));
					$scope.filterData.list = '';
				};

				$scope.addMultiListRow = function() {

					$scope.listTitle = LANGUAGE.translate("Databases");
					$scope.listData = $scope.saveData.list.join("\n");

					$scope.listUIB = $uibModal.open({
						animation: true,
						scope: $scope,
						ariaLabelledBy: 'modal-title',
						ariaDescribedBy: 'modal-body',
						templateUrl: './app/views/listSelection.ptt'
					});

					$scope.listUIB.result.then(function(records) {
						while($scope.saveData.list.length) $scope.saveData.list.pop();
						records = records.split("\n");
						var invalidNames = [];
						for(var i = 0; i < records.length; i++) {
							if($scope.validateName(records[i])) $scope.saveData.list.push(records[i]);
							else invalidNames.push(records[i]);
						}

						if(invalidNames.length) SweetAlert.error(LANGUAGE.translate("Error!"), LANGUAGE.translate("The following databases (\"%s\") has invalid name.", invalidNames.join(", ")))
					}, function() {});
				};

				$scope.rangeStartValueCheck = function (char) {
					return char <= $scope.saveData.rangeend;
				};

				$scope.rangeEndValueCheck = function (char) {
					return char >= $scope.saveData.rangestart;
				};

				$scope.fetchFilterData = function(filter_id) {
					jetapi.getDatabaseFilter({ _id: filter_id }, function(response) {
						$scope.details = response.data;
						$scope.loadFilterType($scope.details.type);
					});
				};

				$scope.$watch("saveData", function(){
					$scope.changed = utils.isChanged($scope.saveData, $scope.details, ['_id']);
				}, true);

				$scope.$on('createFilter', function (event) {
					$scope.saveChanges(true, function(response) {
						$scope.$emit('filterResponse', response);
					});
				});

				$scope.$on('$destroy', function() {
					if($scope.changed && !$scope.cancelled && !$scope.popup && !$scope.wizardData)
					{
						SweetAlert.swal({
							title: LANGUAGE.translate("Are you sure?"),
							text: LANGUAGE.translate("You didn't saved your changes"),
							icon: "warning",
							buttons: [LANGUAGE.translate("Disregard Changes"), LANGUAGE.translate("Save")]
						}, function(isConfirm){
							if (isConfirm) {
								$scope.saveChanges();
							}
						});
					}
				});

				$scope.cancel = function() {
					$scope.cancelled = true;
					$location.path('/databaseFilters');
				};

				$scope.saveChanges = function(apply, callback) {

					if($scope.saveing) return;

					$scope.status = undefined;
					$scope.saveing = true;

					var apiParams = utils.saveParams($scope.saveData, $scope.details, [], function(key) {
						return ($scope.details._id === undefined && key === 'type');
					});

					apiParams.action = $scope.details._id ? 'modify' : 'create';

					jetapi.manageDatabaseFilter(apiParams, function(response) {

						$scope.saveing = false;

						if(callback !== undefined && typeof callback === 'function') {
							callback(response);
							return;
						}

						if(response.success)
						{
							$scope.changed = false;
							$scope.saveData._id = response.data._id;
							$scope.details = utils.duplicateObject($scope.saveData);
							if(apply) $location.path('/databaseFilterManage/' + response.data._id);
							else $location.path('/databaseFilters');

							SweetAlert.swal(LANGUAGE.translate("Success!"), LANGUAGE.translate(response.message), "success");
						}
						else
						{
							$scope.status = {
								message: LANGUAGE.translate(response.message),
								type: response.success ? "success" : "danger",
								closeable: false,
								ttl: 10000
							};
						}
					});
				};

				var filter_id = $routeParams.id;
				if(filter_id) $timeout($scope.fetchFilterData(filter_id));
				else $scope.loadFilterType(CONSTS.DATABASE_FILTER_TYPE_DATABASE, true);
	}]);
});