'use strict';

define([
	'app',
	'libraries/language',
	'libraries/permissions'
	], function(app, LANGUAGE, PERMISSIONS) {
	app.controller("databaseManage",
		["$uibModal", "$rootScope", "$scope", "$routeParams", "$location", "$timeout", "jetapi", "meta", "utils", "SweetAlert",
			function ($uibModal, $rootScope, $scope, $routeParams, $location, $timeout, jetapi, meta, utils, SweetAlert) {
				$rootScope.$emit('menuItem', 'Databases');

				$scope.details = {
					name: '',
					type: '',
					authentication: {},
					options: {}
				};

				$scope.saveing = false;

				$scope.status = undefined;
				$scope.changed = false;
				$scope.cancelled = false;

				$scope.types = [
					{ label: "MySQL", value: 'MySQL' },
					{ label: "PostgreSQL", value: 'PostgreSQL' },
					{ label: "MongoDB", value: 'MongoDB' }
				];

				$scope.gtidpurged_options = [
					{ label: LANGUAGE.translate("Default"), value: '' },
					{ label: "Disabled", value: 'OFF' },
					{ label: "Enabled", value: 'ON' },
					{ label: "Automatically", value: 'AUTO' }
				];

				$scope.mysqlpackets_options = [
					{ label: LANGUAGE.translate("Default"), value: 0 },
					{ label: "32 MB", value: 32 },
					{ label: "64 MB", value: 64 },
					{ label: "128 MB", value: 128 },
					{ label: "256 MB", value: 256 },
					{ label: "512 MB", value: 512 },
					{ label: "1 GB", value: 1024 },
					{ label: "2 GB", value: 2048 }
				];

				$scope.isFlagChecked = function(flag) {
					return !!(flag & $scope.saveData.options.flags);
				};

				$scope.changeFlagState = function(flag, state) {
					if(!state) $scope.saveData.options.flags ^= flag;
					else $scope.saveData.options.flags |= flag;
				};

				$scope.toggleFlagState = function(flag) {
					$scope.changeFlagState(flag, !$scope.isFlagChecked(flag));
				};

				$scope.loadDatabaseType = function(type, defaults) {

					$scope.saveData = utils.duplicateObject($scope.details);

					if(defaults)
					{
						$scope.saveData.type = type;

						switch(type) {
							case 'MySQL':
								$scope.saveData.authentication.hostname = 'localhost';
								$scope.saveData.authentication.port = 3306;
								$scope.saveData.authentication.username = 'root';
								$scope.saveData.options.dump_max_packets = 0;
								$scope.saveData.options.dump_gtid_purged = '';
								$scope.saveData.options.flags = 0;
							break;


							case 'PostgreSQL':
								$scope.saveData.authentication.enabled = 1;
								$scope.saveData.authentication.hostname = 'localhost';
								$scope.saveData.authentication.port = 5432;
								$scope.saveData.authentication.username = 'postgres';
								break;

							case 'MongoDB':
								$scope.saveData.authentication.enabled = 1;
								$scope.saveData.authentication.hostname = 'localhost';
								$scope.saveData.authentication.port = 27017;
								$scope.saveData.authentication.username = '';
								$scope.saveData.authentication.database = 'admin';
							break;
						}
					}

					$scope.databaseType = './app/views/databases/' + type + '.ptt';
				};

				$scope.fetchDatabaseData = function(database_id) {

					jetapi.getDatabaseServer({ _id: database_id }, function(response) {
						$scope.details = response.data;
						$scope.loadDatabaseType($scope.details.type);
					});

				};

				$scope.$watch("saveData", function(){
					$scope.changed = utils.isChanged($scope.saveData, $scope.details, ['_id']);
				}, true);

				$scope.$on('$destroy', function() {
					if($scope.jobWizard) return;
					if($scope.changed && !$scope.cancelled)
					{
						SweetAlert.swal({
							title: LANGUAGE.translate("Are you sure?"),
							text: LANGUAGE.translate("You didn't saved your changes"),
							icon: "warning",
							buttons: [LANGUAGE.translate("Disregard Changes"), LANGUAGE.translate("Save")]
						}, function(isConfirm){
							if (isConfirm) {
								$scope.saveChanges();
							}
						});
					}
				});

				$scope.$on('createDatabase', function (event) {
					$scope.saveChanges(true, function(response) {
						$scope.$emit('databaseResponse', response);
					});
				});

				$scope.cancel = function() {
					$scope.cancelled = true;
					$location.path('/databases');
				};

				$scope.saveChanges = function(apply, callback) {

					if($scope.saveing) return;

					$scope.status = undefined;
					$scope.saveing = true;

					var apiParams = utils.saveParams($scope.saveData, $scope.details);
					apiParams.action = $scope.details._id ? 'modify' : 'create';

					jetapi.manageDatabaseServer(apiParams, function(response) {

						$scope.saveing = false;
						if(callback !== undefined && typeof callback === 'function') callback(response);

						if(response.success)
						{
							$scope.changed = false;
							$scope.saveData._id = response.data._id;
							$scope.details = utils.duplicateObject($scope.saveData);

							if(callback === undefined) {
								if(apply) $location.path('/databaseManage/' + response.data._id);
								else $location.path('/databases');
								SweetAlert.swal(LANGUAGE.translate("Success!"), LANGUAGE.translate(response.message), "success");
							}

						}
						else
						{
							$scope.status = {
								message: LANGUAGE.translate(response.message),
								type: response.success ? "success" : "danger",
								closeable: false,
								ttl: 10000
							};
						}
					});
				};

				$timeout(function () {
					if($routeParams.id) $scope.fetchDatabaseData($routeParams.id);
					else $scope.loadDatabaseType('MySQL', true);
				});


	}]);
});