'use strict';

define([
	'app',
	'libraries/language',
	'libraries/permissions'
	], function(app, LANGUAGE, PERMISSIONS) {
	app.controller("destinationManage",
		["$uibModal", "$rootScope", "$scope", "$routeParams", "$location", "$timeout", "jetapi", "meta", "utils", "SweetAlert",
			function ($uibModal, $rootScope, $scope, $routeParams, $location, $timeout, jetapi, meta, utils, SweetAlert) {
				$rootScope.$emit('menuItem', 'Destinations');

				$scope.details = {
					type: 'Local',
					disklimit: 95,
					path: '/',
					engine: 1,
					schedules: [],
					time_parsed: '12:00 AM',
					time: 0,
					checked: true
				};

				$scope.saveing = false;
				//$scope.saveData = utils.duplicateObject($scope.details);

				$scope.status = undefined;
				$scope.changed = false;
				$scope.cancelled = false;

				$scope.types = [
					{ label: "Local", value: 'Local', group: LANGUAGE.translate('Backup Destinations') },
					{ label: "SSH", value: 'SSH', group: LANGUAGE.translate('Backup Destinations') },
					{ label: "SFTP", value: 'SFTP', group: LANGUAGE.translate('Backup Destinations') },
					{ label: "FTP", value: 'FTP', group: LANGUAGE.translate('Backup Destinations') },
					{ label: "Rsync.net", value: 'RsyncNet', group: LANGUAGE.translate('Backup Destinations') },
					{ label: "Amazon S3", value: 'AmazonS3', group: LANGUAGE.translate('Backup Destinations') },
					{ label: "Google Drive", value: 'GoogleDrive', group: LANGUAGE.translate('Backup Destinations') },
					{ label: "Dropbox", value: 'Dropbox', group: LANGUAGE.translate('Backup Destinations') },
					{ label: "Backblaze", value: 'Backblaze', group: LANGUAGE.translate('Backup Destinations') },
					//{ label: "SSH", value: 'BMR', group: LANGUAGE.translate('Disaster Recovery Destinations') },
				];

				if(!PERMISSIONS.isRoot) {
					$scope.details.type = 'SSH';
					$scope.types.splice(0, 1);
				}

				$scope.disklimits = [
					{ label: LANGUAGE.translate("Disabled"), value: 0 },
					{ label: "50%", value: 50 },
					{ label: "55%", value: 55 },
					{ label: "60%", value: 60 },
					{ label: "65%", value: 65 },
					{ label: "70%", value: 70 },
					{ label: "75%", value: 75 },
					{ label: "80%", value: 80 },
					{ label: "85%", value: 85 },
					{ label: "90%", value: 90 },
					{ label: "95%", value: 95 }
				];

				$scope.toggleEngine = function(details) {
					if(details === undefined) return;
					if(details.checked) details.engine = 1;
					return details.checked;
				};

				$scope.loadDestinationType = function(type, defaults) {

					$scope.saveData = utils.duplicateObject($scope.details);

					if($scope.jobWizard && !$scope.wizardData.destinationId){
						$scope.saveData.name = 'Destination_' + parseInt(new Date().getTime() % 10000);
					}

					if(defaults)
					{
						$scope.saveData.type = type;

						switch(type)
						{
							case 'Local':
								$scope.saveData.path = '/';
								$scope.saveData.rsyncbwlimit = 0;
								break;

							case 'BMR':
							case 'SSH':
							case 'RsyncNet':
								$scope.saveData.path = './';
								$scope.saveData.port = 22;
								$scope.saveData.timeout = 30;
								$scope.saveData.rsyncbwlimit = 0;
								$scope.saveData.rsyncpreferip = 0;
								$scope.authtype = 1;
								break;

							case 'SFTP':
								$scope.saveData.timeout = 30;
								$scope.saveData.retries = 5;
								$scope.saveData.connections = 2;
								$scope.saveData.port = 22;
								$scope.saveData.path = './';
								break;


							case 'FTP':
								$scope.saveData.timeout = 30;
								$scope.saveData.retries = 5;
								$scope.saveData.connections = 2;
								$scope.saveData.port = 21;
								$scope.saveData.ssl = false;
								$scope.saveData.passive = true;
								break;

							case 'AmazonS3':
								$scope.saveData.path = '';
								$scope.saveData.concurrent = 10;
								$scope.saveData.bandwidth = 0;
								break;

							case 'GoogleDrive':
								$scope.saveData.path = '';
								$scope.saveData.authtype = 1;
								break;

							case 'Backblaze':
								$scope.saveData.path = '';
								$scope.saveData.retries = 3;
								$scope.saveData.retrytimeout = 1;
								break;

							case 'Dropbox':
								$scope.saveData.path = '';
								break;
						}
					}

					$scope.destinationType = './app/views/destinations/' + type + '.ptt';
					
					if(type == "SSH" || type == "BMR"){
						$scope.authtypeChange = function(v){
							if(v == 1) $scope.saveData.password = ""; 
							else $scope.saveData.privatekey = "";
						};
						
						$scope.authtype = $scope.saveData.privatekey ? 1 : 0;
						$scope.authtypeChange($scope.authtype);
					}
				};

				$scope.fetchDestinationData = function(destination_id) {

					jetapi.getDestination({ '_id': destination_id }, function(response) {
						$scope.details = response.data;
						$scope.details.checked = $scope.details.engine === 1;

						//$scope.saveData = utils.duplicateObject($scope.details);
						$scope.loadDestinationType($scope.details.type);
					});

				};

				$scope.$watch("saveData", function(){
					// console.log(utils.isChanged($scope.saveData, $scope.details, ['_id']));
					$scope.changed = utils.isChanged($scope.saveData, $scope.details, ['_id']);
				}, true);

				$scope.$on('$destroy', function() {
					if($scope.jobWizard) return;
					if($scope.changed && !$scope.cancelled)
					{
						SweetAlert.swal({
							title: LANGUAGE.translate("Are you sure?"),
							text: LANGUAGE.translate("You didn't saved your changes"),
							icon: "warning",
							buttons: [LANGUAGE.translate("Disregard Changes"), LANGUAGE.translate("Save")]
						}, function(isConfirm){
							if (isConfirm) {
								$scope.saveChanges();
							}
						});
					}
				});

				$scope.cancel = function() {
					$scope.cancelled = true;
					$location.path('/destinations');
				};

				$scope.parseTime = function (time) {
					var matches = /^([0-9]+):([0-9]+)\s+(AM+|PM+)$/g.exec(time);
					if(!matches) return 0;
					var output = parseInt(matches[1] + matches[2]);
					if(output >= 1200) output -= 1200;
					if(matches[3] === 'PM') output += 1200;
					return output;
				};

				$scope.saveChanges = function(apply, callback) {

					if($scope.saveing) return;

					$scope.status = undefined;
					$scope.saveing = true;

					var apiParams = utils.saveParams($scope.saveData, $scope.details, ['checked'], function(key) {
						return ($scope.details._id === undefined && key === 'type');
					});

					if(apiParams.time_parsed) apiParams.time = $scope.parseTime(apiParams.time_parsed);
					apiParams.action = $scope.details._id ? 'modify' : 'create';

					jetapi.manageDestination(apiParams, function(response) {

						$scope.saveing = false;
						if(callback !== undefined && typeof callback === 'function') callback(response);

						if(response.success)
						{
							$scope.changed = false;
							$scope.saveData._id = response.data._id;
							$scope.details = utils.duplicateObject($scope.saveData);

							if(callback === undefined) {
								if(response.data.region !== undefined) $scope.details.region = response.data.region;
								if(apply) $location.path('/destinationManage/' + response.data._id);
								else $location.path('/destinations');
								SweetAlert.swal(LANGUAGE.translate("Success!"), LANGUAGE.translate(response.message), "success");
							}

						}
						else
						{
							$scope.status = {
								message: LANGUAGE.translate(response.message),
								type: response.success ? "success" : "danger",
								closeable: false,
								ttl: 10000
							};
						}
					});
				};

				//Handle backupJobWizard broadcast
				$scope.$on($scope.wizardName, function () {
					$scope.saveData['disasterrecovery'] = true;
					$scope.saveChanges(false, function (response) {
					$scope.process(response);
					});
				});

				$timeout(function () {
					if ($scope.jobWizard) $routeParams.id = $scope.wizardData.destinationId;
					if($routeParams.id) $scope.fetchDestinationData($routeParams.id);
					else $scope.loadDestinationType($scope.details.type, true);
				});


	}]);
});