'use strict';

define([
		'app',
		'libraries/consts',
		'libraries/utils',
		'libraries/language'
	], function(app, CONSTS, UTILS, LANGUAGE) {

		app.controller("fileBrowse",
			["$uibModalInstance", "$uibModal", "$scope", "$interval", "$timeout", "jetapi", "meta", "SweetAlert", "pathManager", "listPaths",
				function($uibModalInstance, $uibModal, $scope, $interval, $timeout, jetapi, meta, SweetAlert, pathManager, listPaths) {

					$scope.manager = pathManager.new(listPaths);

					$scope.files = [];
					$scope.selectedTree = {};
					$scope.loadingFiles = false;
					$scope.breadcrumbs = [];
					$scope.filesIndex = {};


					$scope.toggleFile = function(file) {
						if($scope.manager.isExists(file.path)) {
							$scope.manager.removePath(file.path);
						} else {
							$scope.manager.addPath(file.path);
						}
					};

					$scope.noParentDirectory = function() {
						return $scope.breadcrumbs.length <= 1;
					};

					$scope.goParentDirectory = function() {
						if($scope.noParentDirectory()) return;
						$scope.fetch($scope.breadcrumbs[$scope.breadcrumbs.length-2]);
					};

					$scope.isDisabled = function(file) {
						return $scope.manager.isChildren(file.path);
					};

					$scope.isChecked = function(file) {
						return $scope.manager.isExists(file.path) || $scope.manager.isChildren(file.path);
					};

					$scope.isChildrenChecked = function(file) {
						return $scope.manager.isParent(file.path);
					};

					var calculateBreadcrumbs = function(file) {
						var path = file.path;
						if(path.substr(-1) === '/') path = path.substr(0, path.length-1);
						if(path.substr(0, 1) === '/') path = path.substr(1);

						var parts = path ? path.split('/') : [];

						$scope.breadcrumbs = [{name:"", path:"/"}];

						var fullpath = '';
						for(var i = 0; i < parts.length; i++) {
							fullpath += '/' + parts[i];
							$scope.breadcrumbs.push({name: parts[i], path: fullpath});
						}
					};

					var handleResponse = function(response, file) {
						$scope.filesIndex[file.path] = response;
						$scope.files = response.data.files;
						$scope.loadingFiles = false;
					};

					$scope.fetch = function(file) {

						if(file.path === undefined) file.path = '/';

						$scope.loadingFiles = true;
						$scope.files = [];

						calculateBreadcrumbs(file);

						if($scope.filesIndex[file.path] !== undefined)
						{
							handleResponse($scope.filesIndex[file.path], file);
							return;
						}

						var apiParams = {};
						apiParams['_id'] = $scope.typeId;
						apiParams['type'] = $scope.type;
						apiParams['path'] = file.path;

						jetapi.fileBrowse(apiParams, function(response) {

							if(!response.success) {
								SweetAlert.error(LANGUAGE.translate("Error!"), LANGUAGE.translate(response.message));
								return false;
							}

							for(var i = 0; i < response.data.files.length; i++) {
								response.data.files[i].path = file.path + (file.path === '/' ? '' : '/') + response.data.files[i].name;
							}

							handleResponse(response, file);
							return true;
						});

					};

					$timeout($scope.fetch({path: '/'}));

					$scope.ok = function () { $uibModalInstance.close($scope.manager.getSelected()); };
					$scope.cancel = function () { $uibModalInstance.dismiss(LANGUAGE.translate('cancel')); };

				}]
		);

	}
);