'use strict';

define([
	'app',
	'libraries/language',
	'libraries/consts'
	], function(app, LANGUAGE, CONSTS) {
	app.controller("hooks",
		["$rootScope", "$scope", "$location", "$timeout", "jetapi", "meta", "SweetAlert",
			function ($rootScope, $scope, $location, $timeout, jetapi, meta, SweetAlert) {
				$rootScope.$emit('menuItem', 'Hooks');

				$scope.hooks = [];
				$scope.loading = false;
				$scope.saveing = false;
				$scope.status = undefined;
				$scope.filter = '';
				$scope.filterOptions = [
					{ label: LANGUAGE.translate('All Hook Positions'), value: '' },
					{ label: LANGUAGE.translate('Pre'), value: CONSTS.HOOK_TYPE_POSITION_PRE },
					{ label: LANGUAGE.translate('Post'), value: CONSTS.HOOK_TYPE_POSITION_POST }
				];

				$scope.loaders = {
					delete: false,
					state: false
				};

				$scope.backups = {};
				$scope.destinations = {};

				$scope.types = {};

				$scope.types[CONSTS.BACKUP_TYPE_DIRECTORIES] = LANGUAGE.translate("Directories");

				$scope.positions = {};
				$scope.positions[CONSTS.HOOK_POSITION_BACKUP] = LANGUAGE.translate("Backup");
				$scope.positions[CONSTS.HOOK_POSITION_RESTORE] = LANGUAGE.translate("Restore");
				$scope.positions[CONSTS.HOOK_POSITION_DOWNLOAD] = LANGUAGE.translate("Download");
				$scope.positions[CONSTS.HOOK_POSITION_DAILY_CRON] = LANGUAGE.translate("Daily Cron");

				$scope.position_types = {};
				$scope.position_types[CONSTS.HOOK_TYPE_POSITION_PRE] = LANGUAGE.translate("Pre");
				$scope.position_types[CONSTS.HOOK_TYPE_POSITION_POST] = LANGUAGE.translate("Post");

				meta = meta.new("hooks");
				$scope.meta = meta;
				$scope.metaData = meta.getData();

				if(!meta.getSortBy()) meta.setSortBy("name");
				if(!meta.getSortDirection()) meta.setSortDirection("asc");
				meta.setSortFields(["name","position_type","position","owner","disabled"]);
				meta.setTotalItems($scope.hooks.length);

				$scope.onClickDelete = function(hook) {

					if($scope.saveing) return;

					$scope.saveing = true;
					$scope.loaders.delete = true;

					SweetAlert.swal({
						title: LANGUAGE.translate("Are you sure?"),
						text: LANGUAGE.translate("This hook will be permanently deleted!"),
						icon: "warning",
						buttons: [LANGUAGE.translate("Cancel"), true]
					}, function(isConfirm){

						if (isConfirm)
						{
							jetapi.deleteHook({ _id: hook._id }, function(response) {

								$scope.saveing = false;
								$scope.loaders.delete = false;

								if(response.success)
								{
									$scope.fetchData();
									SweetAlert.success(LANGUAGE.translate("Success!"), LANGUAGE.translate(response.message));
								}
								else
								{
									SweetAlert.error(LANGUAGE.translate("Failed!"), LANGUAGE.translate(response.message));
								}

							});
						}
						else
						{
							$scope.saveing = false;
							$scope.loaders.delete = false;
							SweetAlert.error(LANGUAGE.translate("Cancelled!"), LANGUAGE.translate("Hook delete cancelled"));
						}

					});
				};

				$scope.toggleStatus = function(hook) {

					$scope.loaders.state = true;

					var disabled = !hook.disabled;

					jetapi.manageHook({
						_id: hook._id,
						action: 'modify',
						disabled: disabled
					}, function(response) {
						$scope.loaders.state = false;
						if(response.success) hook.disabled = disabled;
					});
				};

				$scope.fetchData = function() {

					$scope.loading = true;
					$scope.status = undefined;

					var apiParams = {
						sort: {},
						skip: meta.getSkip(),
						limit: meta.getPageSize(),
						find: { position_type: $scope.filter },
						filter: meta.getFilter()
					};

					apiParams.sort[meta.getSortBy()] = meta.getSortDirectionInt();

					$scope.hooks = [];

					jetapi.listHooks(apiParams, function(response) {
						meta.setTotalItems(response.data.total);
						meta.calculate(response.data.hooks);

						for(var i = 0; i < response.data.hooks.length; i++) {

							var list = response.data.hooks[i].list;

							if(list.length) {
								switch(response.data.hooks[i].position) {
									case CONSTS.HOOK_POSITION_BACKUP:
										for(var j = 0; j < list.length; j++) list[j] = ($scope.backups[list[j]] !== undefined) ? $scope.backups[list[j]].name : 'Unknown';
										break;

									case CONSTS.HOOK_POSITION_RESTORE:
									case CONSTS.HOOK_POSITION_DOWNLOAD:
										for(var j = 0; j < list.length; j++) list[j] = ($scope.types[list[j]] !== undefined) ? $scope.types[list[j]] : 'Unknown';
										break;
								}
							}

							$scope.hooks.push(response.data.hooks[i]);

						}

						$scope.loading = false;
					});
				};

				$scope.fetch = $scope.fetchData;
				$scope.$on('fetch', $scope.fetch());

				jetapi.listBackupJobs({}, function(response) {
					for(var i = 0; i < response.data.jobs.length; i++) $scope.backups[response.data.jobs[i]._id] = response.data.jobs[i];
				});
			}
		]
	);
});