'use strict';

define([
	'app',
	'libraries/language',
	'libraries/permissions'
	], function(app, LANGUAGE, PERMISSIONS) {
	app.controller("settings",
		["$rootScope", "$scope", "$routeParams", "$location", "$timeout", "jetapi", "meta", "SweetAlert", "utils",
			function ($rootScope, $scope, $routeParams, $location, $timeout, jetapi, meta, SweetAlert, utils) {
				$rootScope.$emit('menuItem', 'Settings');

				$scope.saveing = false;
				$scope.status = undefined;
				$scope.changed = false;
				$scope.loading = false;

				$scope.details = {};
				$scope.saveData = {};

				$scope.sections = [
					{ _id: 'general', name: LANGUAGE.translate('General'), template: 'general.ptt', icon: 'fa-cogs' },
					{ _id: 'performance', name: LANGUAGE.translate('Performance'), template: 'performance.ptt', icon: 'fa-sliders-h' },
					{ _id: 'resource', name: LANGUAGE.translate('Resource'), template: 'resource.ptt', icon: 'fa-microchip' },
					//{ _id: 'restore', name: LANGUAGE.translate('Restore'), template: 'restore.ptt', icon: 'fa-sync' },
					{ _id: 'security', name: LANGUAGE.translate('Security'), template: 'security.ptt', icon: 'fa-shield-alt' },
					{ _id: 'notification', name: LANGUAGE.translate('Notification'), template: 'notification.ptt', icon: 'fa-bell' },
					{ _id: 'binary', name: LANGUAGE.translate('Binary Locations'), template: 'binary.ptt', icon: 'fa-terminal' }
				];

				$scope.getSection = function (section_id) {
					for(var i = 0; i < $scope.sections.length; i++) {
						if($scope.sections[i]._id === section_id) {
							return $scope.sections[i];
						}
					}

					return $scope.sections[0];
				};

				$scope.changeSection = function (section) {

					if($scope.changed)
					{
						SweetAlert.swal({
							title: LANGUAGE.translate("Are you sure?"),
							text: LANGUAGE.translate("You didn't saved your changes"),
							icon: "warning",
							buttons: [LANGUAGE.translate("Disregard Changes"), LANGUAGE.translate("Save")]
						}, function(isConfirm){
							if (isConfirm) {
								$scope.saveChanges(function () {
									$location.path('/settings/' + section._id)
								});
							} else $location.path('/settings/' + section._id);
						});
					}
					else $location.path('/settings/' + section._id);
				};

				$scope.$on('$destroy', function() {
					if($scope.changed)
					{
						SweetAlert.swal({
							title: LANGUAGE.translate("Are you sure?"),
							text: LANGUAGE.translate("You didn't saved your changes"),
							icon: "warning",
							buttons: [LANGUAGE.translate("Disregard Changes"), LANGUAGE.translate("Save")]
						}, function(isConfirm){
							if (isConfirm) {
								$scope.saveChanges();
							}
						});
					}
				});

				$scope.loadSection = function (section) {
					$scope.currentSection = section;
					$scope.settingsSection = './app/views/settings/' + section.template;

					if(section.standAlone) {
						$rootScope.$emit('menuItem', section.class);
					} else {
						$rootScope.$emit('menuItem', 'Settings');
					}
				};

				$scope.loadSection($scope.getSection($routeParams.section ? $routeParams.section : 'general'));

				$scope.$watch("saveData", function(){
					$scope.changed = utils.isChanged($scope.saveData, $scope.details);
				}, true);

				$scope.saveChanges = function(callback) {

					if($scope.saveing) return;

					$scope.status = undefined;
					$scope.saveing = true;

					var apiParams = utils.saveParams($scope.saveData, $scope.details, undefined, function (key) {
						return key == 'mustagree' || key == 'suspendbackups';
					});
					apiParams.section = $scope.currentSection._id;

					jetapi.manageSettings(apiParams, function(response) {

						$scope.saveing = false;

						if(response.success)
						{
							$scope.changed = false;
							$scope.saveData._id = response.data._id;
							$scope.details = utils.duplicateObject($scope.saveData);
							if(callback !== undefined) callback();
							SweetAlert.swal(LANGUAGE.translate("Success!"), LANGUAGE.translate(response.message), "success");
						}
						else
						{
							$scope.status = {
								message: LANGUAGE.translate(response.message),
								type: "danger",
								closeable: false,
								ttl: 10000
							};
						}
					});
				};

				$scope.fetch = function() {
					$scope.loading = true;
					jetapi.getSettings({ section: $scope.currentSection._id }, function(response) {
						$scope.loading = false;
						$scope.details = response.data;
						$scope.saveData = utils.duplicateObject($scope.details);
					});
				};

				$timeout($scope.fetch());

			}]);
});