'use strict';

define([
	'app',
	'libraries/language'
], function (app, LANGUAGE) {

	app.directive("alertBox", [
		function() {

			var LABELS = [{
				name: "errorLabel",
				defaultText: LANGUAGE.translate("Error:")
			}, {
				name: "warnLabel",
				defaultText: LANGUAGE.translate("Warning:")
			}, {
				name: "infoLabel",
				defaultText: LANGUAGE.translate("Information:")
			}, {
				name: "successLabel",
				defaultText: LANGUAGE.translate("Success:")
			}, {
				name: "moreLabel",
				defaultText: LANGUAGE.translate("What went wrong?")
			}];

			var initializeModel = function(hasModel, attrs, modelValue) {
				var data = {};

				if (hasModel) {
					if (angular.isString(modelValue)) {
						data.message = modelValue;
					} else if (angular.isObject(modelValue)) {
						angular.copy(modelValue, data);
					} else {
						throw new TypeError("ngModel must be a string or object.");
					}
				}

				if (!angular.isDefined(data.type)) {
					if (angular.isDefined(attrs.type) && attrs.type) {
						data.type = attrs.type;
					} else {
						data.type = "warning";
					}
				}

				if (hasModel && !angular.isDefined(data.message) && !data.message) {
					throw new Error("No message provided in the model's message property.");
				}

				return data;
			};

			var renderBody = function(scope, element, transclude) {

				var type = scope.alert.type;
				var typeBlock = element[0].querySelector(".alert-" + type);
				var messageSpan = typeBlock.querySelector(".alert-body");

				transclude(function(clone) {

					angular.element(messageSpan).append(clone);
				});
			};


			return {
				restrict: "EA",
				template: '<div>\n' +
					'    <div class=\'alert alert-{{alert.type}}\'>\n' +
					'        <div class=\'alert-message\'>\n' +
					'        <span class=\'fas {{alert.icon}}\'></span>\n' +
					'            <span class="alert-body"><span ng-bind-html="alert.message" ng-if="alert && alert.message"></span></span>\n' +
					'            <ul ng-if="alert.list && alert.list.length" class="alert-list">\n' +
					'                <li ng-repeat="value in alert.list">\n' +
					'                    <span ng-bind-html="value"></span>\n' +
					'                </li>\n' +
					'            </ul>\n' +
					'        </div>\n' +
					'    </div>',
				transclude: true,
				replace: true,
				require: "?ngModel",
				scope: {
				},
				compile: function(element, attrs) {

					LABELS.forEach(function(label) {
						if (!angular.isDefined(attrs[label.name])) {
							attrs[label.name] = label.defaultText;
						}
					});

					return function(scope, element, attrs, ngModelCtrl, transclude) {

						if (ngModelCtrl) {
							ngModelCtrl.$formatters.push(function(modelValue) {
								return initializeModel(true, attrs, modelValue);
							});

							ngModelCtrl.$render = function() {
								scope.alert = ngModelCtrl.$viewValue;

								switch(scope.alert.type) {
									case 'danger': scope.alert.icon = "fa-times-circle"; break;
									case 'success': scope.alert.icon = "fa-check-circle"; break;
									case 'info': scope.alert.icon = "fa-info-circle"; break;
									case 'warning': scope.alert.icon = "fa-exclamation-triangle"; break;
								}
							};
						} else {
							scope.alert = initializeModel(false, attrs);
							renderBody(scope, element, transclude);
						}

						scope.$watch("alert.label", function(newVal) {
							if ( angular.isDefined(newVal) ) {
								LABELS.forEach(function(label) {
									attrs.$set(label.name, newVal);
								});
							}
						});
					};
				}
			};
		}
	]);

});