'use strict';

define([], function() {
	var PERMISSIONS = function() { this._init(); };

	PERMISSIONS.prototype = {
		_permissions: {},
		_account: '',
		_licenseText: '',
		_init: function () {

			this._permissions = window.PAGE.permissions;
			this._account = window.PAGE.account;

			this.hasAllPrivileges = this._can(0);
			this.isRoot = this._account === 'root' || this.hasAllPrivileges;

			this.isLoggedIn = !!this._account.loggedIn;
			this.isOTP = !!this._account.otp;

			this.isShowcase = this.isLoggedIn && this.isRoot && window.PAGE.showcase.total_unapproved;
			this.isLicenseIssue = this.isLoggedIn && window.PAGE.system.licenseIssue;
			this.isDisasterRecovery = this.isLoggedIn && window.PAGE.system.drMode;
			this.isAgreement = this.isLoggedIn && window.PAGE.system.agreement;

			this.isFullscreen = !this.isLoggedIn || this.isLicenseIssue || this.isDisasterRecovery || this.isShowcase || this.isAgreement;

			this.canManageDestinations = this._can(1);
			this.canManageBackupJobs = this._can(2);
			this.canManageBackups = this._can(3);
			this.canManageSchedules = this.canManageBackupJobs || this.canManageDestinations;
			this.canManageQueue = this.canManageBackups || this.canManageDestinations;
			this.canManageFileManager = this.canManageBackups || this.canManageDestinations;
		},
		_can: function (permission) {
			return !!this._permissions[permission];
		},
		_get: function(permission) {

			var permissions = {
				0: 'Grant All Privileges (same as root)',
				1: 'Can manage destinations',
				2: 'Can manage backup jobs',
				3: 'Can manage backups',
			};

			return permissions[permission] !== undefined ? permissions[permission] : null;
		}
	};

	return new PERMISSIONS();
});