
'use strict';

define([
	'app',
	'libraries/consts',
	'libraries/language'
], function (app, CONSTS, LANGUAGE) {
	app.factory('JWizardService', ['$rootScope' ,'storage', '$uibModal', 'SweetAlert', function ($rootScope, storage, $uibModal, SweetAlert) {


		var WIZARD = function (storageName) {
			if(!storageName || storageName === undefined) return; //TODO - throw error
			const wName = storageName+"_JWizard";
			// if(storageName === undefined) storageName = CONSTS.WIZARD_STORAGE;//TODO remove this const.
			this._init(wName);
		};

		WIZARD.prototype = {

			_wizardStorage: null,
			_wizardName: null,
			_inProgress: false,
			_model: null,
			_steps: {},

			_stepsChain: [],
			_firstStep: null,

			currentStep: null,
			wizardAlert: null,
			data:{},
			isSaving: false,
			finish: function() {},

			setWatch: function(callback) {
				this.watch = callback;
			},

			watch: function() {},

			_init: function(storageName) {
				this._wizardName = storageName;
				this._inProgress = storage.exists(this._wizardName);
				this._storageConnect();
			},

			_openWizard: function () {
				var self = this;
				this._model = $uibModal.open({
					animation: true,
					size: "lg",
					ariaLabelledBy: 'modal-title',
					ariaDescribedBy: 'modal-body',
					templateUrl: './app/views/wizard/JWizard.ptt',
					controller: 'JWizard',
					resolve: {
						wizardStorage: function () { return self._wizardStorage;},
						JWService: function () { return self; }
					}
				});
			},

			_storageConnect: function(){
				this._wizardStorage = storage.create(this._wizardName);
			},

			cleanStorageData: function() {
				// storage.delete(this._wizardName);
				this._wizardStorage.destroy();
				this._inProgress = false;
			},

			_updatedSteps: function(next) {
				this._stepsChain.push(this.currentStep.name);
				this.updateStorage('stepsChain', this._stepsChain);
				this.currentStep = this._steps[next];
				this.updateStorage('currStep', this.currentStep);
			},

			_updateFromStorage: function() {
				var step = this._wizardStorage.get("currStep", this._firstStep);
				this.currentStep = this._steps[step.name];
				this._stepsChain = this._wizardStorage.get("stepsChain", []);
			},

			updateStorage: function (key, val) {
				this._wizardStorage.set(key, val);
				this._wizardStorage.save();
				this.watch();
			},

			getData:function() {
				return this._wizardStorage.getData();
			},

			setFirstStep: function(step) {
				this._firstStep = step.name;
				this.addStep(step);
				if(!this.currentStep) this.currentStep = this._steps[this._firstStep];
			},

			addStep: function(step){
				this._steps[step.name] = step;
			},

			next:function() {
				$rootScope.$broadcast(this._wizardName, this.getData());
			},

			prev:function() {
				if (this._stepsChain.length < 1) return;
				this.currentStep = this._steps[this._stepsChain.pop()];
			},

			/**
			 * This is the main functionality of the wizard -
			 * This must return an Object that has {success:boolean data:any}
			 * this.currentStep.process @returns Next step.
			 * @param data
			 */
			process: function (data) {
				if(!data.success) return;

				var nextStep = this.currentStep.process(data);
				if(this.currentStep.final) { this.finish(); return; }

				if(!nextStep) return;

				this._updatedSteps(nextStep);
			},

			setFinish: function(finish){
				this.finish = finish;
			},

			open: function () {
				var self = this;
				if(this._inProgress) {
					SweetAlert.swal({
						title: LANGUAGE.translate("Uncompleted Wizard Process "),
						text: LANGUAGE.translate("You have an uncompleted Wizard Process. Would you like to continue?"),
						icon: "warning",
						buttons: [LANGUAGE.translate("Start Over"), LANGUAGE.translate("YES")]
					}, function (isConfirm) {
						if (!isConfirm) {
							self.cleanStorageData();
							self._storageConnect();
						}
						else self._updateFromStorage();

						self._openWizard();
					});
				}
				else self._openWizard();
			},
			close: function () {
				if(this._model === null) return;
				this._model.dismiss();
			},
			hide: function () {
				if(this._model === null) return;
				this._model.hide();
			}
		};

		return WIZARD;

	}]);
});