'use strict';

define(['app'], function (app) {
	app.factory('pathManager', [function () {

		var pathManager = function(list) { this._init(list); };

		pathManager.prototype = {
			_tree: {},
			_init: function(list) {
				this._tree = {};
				if(list !== undefined) for(var i = 0; i < list.length; i++) this.addPath(list[i]);
			},

			_getCurrent: function(path) {
				var parts = path.split('/');
				var current = this.getTree();
				while (parts.length) {
					var part = parts.shift();
					if(!part) continue;
					if(current[part] === undefined) return false;
					current = current[part];
				}
				return current;
			},
			_countObj: function (obj) {
				var count = 0;
				for(var key in obj) count++;
				return count;
			},
			isExists: function(path) {
				if(!this._countObj(this.getTree())) return false;
				var current = this._getCurrent(path);
				if(!current) return false;
				return !this._countObj(current);
			},
			isParent: function(path) {
				if(!this._countObj(this.getTree())) return false;
				var current = this._getCurrent(path);
				if(!current) return false;
				return !!this._countObj(current);
			},
			isChildren: function(path) {
				if(!this._countObj(this.getTree())) return false;
				var parts = path.split('/');
				var current = this.getTree();
				while (parts.length) {
					var part = parts.shift();
					if(!part) continue;
					if(current[part] === undefined) return !this._countObj(current);
					current = current[part];
				}
				return false;
			},
			addPath: function (path) {


				console.log(path);
				var parts = path.split('/');
				var current = this.getTree();
				while (parts.length) {
					var part = parts.shift();
					if(!part) continue;
					if(current[part] === undefined) current[part] = {};
					current = current[part];
				}

				// remove sub folders
				for(var key in current) delete current[key];
			},
			removePath: function (path) {
				// TODO remove from tree
				var parts = path.split('/');
				var current = this.getTree();
				var queue = [];
				while (parts.length) {
					var part = parts.shift();
					if(!part) continue;
					if(current[part] === undefined) return false;
					queue.push({ instance: current, name: part });
					current = current[part];
				}

				if(!queue.length) return;

				var item = queue.pop();
				delete item.instance[item.name];

				while(queue.length) {
					item = queue.pop();
					console.log(this.getTree());
					if(this._countObj(item.instance[item.name]) > 0) return;
					delete item.instance[item.name];
				}

			},
			getSelected: function () {

				var paths = [];
				var tree = this.getTree();
				var queue = [{path: '', obj: tree}];

				while(queue.length) {
					var item = queue.shift();

					for(var key in item.obj) {

						var path = item.path + '/' + key;
						if(this._countObj(item.obj[key])) {
							queue.push({path: path, obj: item.obj[key]});
						} else {
							paths.push(path);
						}
					}
				}

				return paths;
			},
			getTree: function () { return this._tree; }
		};

		return { new: function (list) { return new pathManager(list); } };
	}]);
});