
'use strict';

define([
	'app',
	'libraries/consts',
	'libraries/language',
	'libraries/permissions'
], function (app, CONSTS, LANGUAGE, PERMISSIONS) {
	app.factory('wizards', ["JWizardService", "SweetAlert", function (JWizardService, SweetAlert) {


		return {


			backupJobs: function (callback) {

				if(callback === undefined) callback = function () {};

				const STEPS = {
					WIZARD_WELCOME: 2123,
					WIZARD_DESTINATION_PICKER: 15,
					WIZARD_DESTINATION_CREATE: 1,
					WIZARD_ENABLE_DR: 2,
					WIZARD_CREATE_BACKUP: 3,
					WIZARD_MANAGE_FILTERS: 4,
					WIZARD_MANAGE_SCHEDULES: 5,
				};

				var wizard = new JWizardService("wizard1");

				if(PERMISSIONS.canManageDestinations) {
					wizard.setFirstStep({
						name: STEPS.WIZARD_WELCOME,
						tpl: 'wizard/wizardWelcomePage',
						title: LANGUAGE.translate("Backup Job Wizard"),
						// description: '<strong>' + LANGUAGE.translate("Please select a backup job to create from the options below") + '</strong>',
						process: function (data) {
							if (data && !data.next) {
								// wizard.updateStorage('jobId', data.jobId._id);
								wizard.finish();
								return;
							}

							return STEPS.WIZARD_DESTINATION_PICKER;
						},
						hideFooter: true
					});

					wizard.addStep({
						name: STEPS.WIZARD_DESTINATION_PICKER,
						tpl: 'wizard/wizardDestinationPicker',
						title: LANGUAGE.translate("Choose Destination"),
						description: LANGUAGE.translate("A Backup destination is the place holder of your backup files (this can be remote or local)") +
							"<br>" + LANGUAGE.translate("For more information please follow: ") +
							"<a class=\"wizard-link\" href=\"https://docs.jetbackup.com/manual/whm/Destinations/createNewDestination.html\" target=\"_blank\">" +
							LANGUAGE.translate("Backup Destinations Documentation") + "</a>",
						process: function (data) {
							wizard.updateStorage('destinationId', data.destination_id);
							return data.destination_id ? STEPS.WIZARD_CREATE_BACKUP : STEPS.WIZARD_DESTINATION_CREATE;
						}
					});
				}
				else {
					wizard.setFirstStep({
						name: STEPS.WIZARD_DESTINATION_PICKER,
						tpl: 'wizard/wizardDestinationPicker',
						title: LANGUAGE.translate("Choose Destination (click next to create new)"),
						description: LANGUAGE.translate("A Backup destination is the place holder of your backup files (this can be remote or local)") +
							"<br>" + LANGUAGE.translate("For more information please follow: ") +
							"<a class=\"wizard-link\" href=\"https://docs.jetbackup.com/manual/whm/Destinations/createNewDestination.html\" target=\"_blank\">" +
							LANGUAGE.translate("Backup Destinations Documentation") + "</a>",
						process: function (data) {
							wizard.updateStorage('destinationId', data.destination_id);
							return data.destination_id ? STEPS.WIZARD_CREATE_BACKUP : STEPS.WIZARD_DESTINATION_CREATE;
						}
					});
				}





				wizard.addStep({
					name: STEPS.WIZARD_DESTINATION_CREATE,
					tpl: 'destinationManage',
					title: LANGUAGE.translate("Manage Destination"),
					description: LANGUAGE.translate("A Backup destination is the place holder of your backup files (this can be remote or local)") +
						"<br>" + LANGUAGE.translate("For more information please follow: ") +
						"<a class=\"wizard-link\" href=\"https://docs.jetbackup.com/manual/whm/Destinations/createNewDestination.html\" target=\"_blank\">" +
						LANGUAGE.translate("Backup Destinations Documentation") + "</a>",
					process: function (data) {
						wizard.updateStorage('destinationId', data.data._id);
						return STEPS.WIZARD_CREATE_BACKUP;
						// return STEPS.WIZARD_ENABLE_DR;
					}
				});

				wizard.addStep({
					name: STEPS.WIZARD_CREATE_BACKUP,
					tpl: 'backupJobManage',
					title: LANGUAGE.translate("Backup Job Settings"),
					description: "<strong>" + LANGUAGE.translate("JetBackup can perform several types of backups:") + "</strong> <br>" +
						LANGUAGE.translate("Databases") + "<br>" +
						LANGUAGE.translate("Directories - Copy server files/folders that you choose to include.") + "<br>" +
						LANGUAGE.translate("Replicate - Copy files/folders \"as is\".") +
						"<br>" + LANGUAGE.translate("For more information please follow: ") +
						"<a class=\"wizard-link\" href=\"https://docs.jetbackup.com/manual/whm/BackupJobs/backupJobTypes.html\" target=\"_blank\">" +
						LANGUAGE.translate("Backup Job") + "</a>",
					process: function (data) {
						wizard.updateStorage('jobId', data.data._id);
						// return data.data.backuptypes <= 127 ? STEPS.WIZARD_MANAGE_FILTERS : STEPS.WIZARD_MANAGE_SCHEDULES;
						return STEPS.WIZARD_MANAGE_SCHEDULES;
					}
				});

				// wizard.addStep({
				// 	name: STEPS.WIZARD_MANAGE_FILTERS,
				// 	tpl: 'wizard/wizardFilters',
				// 	title: LANGUAGE.translate("Select Account Filters or press next to backup all accounts"),
				// 	description: LANGUAGE.translate("A account filter helps us to backup accounts by predefined conditions.") +
				// 		"<br>" + LANGUAGE.translate("For more information please follow: ") +
				// 		"<a class=\"wizard-link\" href=\"https://docs.jetbackup.com/manual/whm/ManageAccountFilters/createNewFilter.html\" target=\"_blank\">" +
				// 		LANGUAGE.translate("Filters") + "</a>",
				// 	process: function (data) {
				// 		wizard.updateStorage('filters', data.data.filters);
				// 		return STEPS.WIZARD_MANAGE_SCHEDULES;
				// 	}
				// });

				wizard.addStep({
					name: STEPS.WIZARD_MANAGE_SCHEDULES,
					tpl: 'wizard/wizardSchedule',
					title: LANGUAGE.translate("Create a Schedule"),
					description: LANGUAGE.translate("A backup job schedule is the date and time that your backup job will be executed.") +
						"<br>" + LANGUAGE.translate("For more information please follow: ") +
						"<a class=\"wizard-link\" href=\"https://docs.jetbackup.com/manual/whm/CreateSchedules/createSchedule.html\" target=\"_blank\">Schedule Documentation</a>",
					process: function (data) {
						wizard.updateStorage('schedules', data.data.schedules);
						wizard.updateStorage('time_parsed', data.data.time_parsed);
					},
					final: true
				});

				var finish = function () {

					if (!wizard.data.jobId) {
						SweetAlert.success(LANGUAGE.translate("Success!"), LANGUAGE.translate("Backup Job Created Successfully"));
						wizard.cleanStorageData();
						wizard.close();
						callback(true);
						return;
					}

					// wizard.next();
					jetapi.manageBackupJob({
						_id: wizard.data.jobId,
						action: 'modify',
						disabled: 0
					}, function (response) {
						wizard.cleanStorageData();
						wizard.close();
						callback(!!response.success);
						if (!response.success) {
							SweetAlert.error(LANGUAGE.translate("Error!"), LANGUAGE.translate(response.message));
							return;
						}
						SweetAlert.success(LANGUAGE.translate("Success!"), LANGUAGE.translate("Backup Job Created Successfully"));
					});
				};

				wizard.setFinish(finish);
				wizard.open();

			}


		};

	}]);
});